using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.IO.Ports;


namespace MultiparameterDecode
{
    public partial class Form1 : Form
    {
        private delegate void nextPackageDeleg(byte[] Package);

        static class Constants
        {
            public const byte ECG_IDENT = 0x45;                 // identifier "E"
            public const byte SPO_IDENT = 0x53;                 // identifier "S"
            public const byte NIBP_IDENT = 0x4e;                // identifier "N"
            public const byte TEMP_IDENT = 0x54;                // identifier "T"
            public const byte CO2_IDENT = 0x43;                 // identifier "C"
            public const byte IBP1_IDENT = 0x49;                // identifier "I"
            public const byte IBP2_IDENT = 0x4A;                // identifier "J"
            public const byte MULTI_IDENT = 0x4d;               // identifier "M"

            public const int ECG_WAVE_ADDRESS = 0x00;
            public const int ECG_NUM_ADDRESS = 0x01;
            public const int ECG_STAT_ADDRESS = 0x02;
            public const int ECG_CMD_ADDRESS = 0x00;

            public const int SPO_WAVE_ADDRESS = 0x00;
            public const int SPO_NUM_ADDRESS = 0x01;
            public const int SPO_STAT_ADDRESS = 0x02;
            public const int SPO_CMD_ADDRESS = 0x01;

            public const int NIBP_CUFF_ADDRESS = 0x10;
            public const int NIBP_NUM_ADDRESS = 0x11;
            public const int NIBP_STAT_ADDRESS = 0x12;
            public const int NIBP_TIMER_ADDRESS = 0x13;
            public const int NIBP_CMD_ADDRESS = 0x02;

            public const int IBP1_WAVE_ADDRESS = 0x05;
            public const int IBP1_INFO1_ADDRESS = 0x06;
            public const int IBP1_INFO2_ADDRESS = 0x07;
            public const int IBP1_STAT_ADDRESS = 0x08;
            public const int IBP1_CMD_ADDRESS = 0x06;

            public const int IBP2_WAVE_ADDRESS = 0x0a;
            public const int IBP2_INFO1_ADDRESS = 0x0b;
            public const int IBP2_INFO2_ADDRESS = 0x0c;
            public const int IBP2_STAT_ADDRESS = 0x0d;
            public const int IBP2_CMD_ADDRESS = 0x07;


            public const int CO2_VALUE_ADDRESS = 0x05;
            public const int CO2_TEMP_ADDRESS = 0x06;
            public const int CO2_AMB_ADDRESS = 0x07;
            public const int CO2_WAVE_ADDRESS = 0x08;
            public const int CO2_RESP_ADDRESS = 0x09;
            public const int CO2_FLOW_ADDRESS = 0x0A;
            public const int CO2_INCO2_ADDRESS = 0x0B;
            public const int CO2_FIO2_ADDRESS = 0x0C;
            public const int CO2_ERR_ADDRESS = 0x0D;
            public const int CO2_INFO_ADDRESS = 0x0E;
            public const int CO2_CMD_ADDRESS = 0x08;


            public const int TEMP_NUM_ADDRESS = 0x20;
            public const int TEMP_STAT_ADDRESS = 0x21;
            public const int TEMP_CMD_ADDRESS = 0x03;

            public const int MULTI_STAT_ADDRESS = 0x30;
            public const int MULTI_VERSION_ADDRESS = 0x31;
            public const int MULTI_NUM_ADDRESS = 0x32;
            public const int MULTI_CMD_ADDRESS = 0x04;
            public const int MULTI_TXD_ADDRESS = 0x05;
            

            public const int CMD_ACK_ADDRESS = 0x40;
            public const int CMD_ERR_ADDRESS = 0x41;
            public const int CMD_TIMEOUT_ADDRESS = 0x42;
            public const int CMD_ERR_CRC_ADDRESS = 0x43;
            public const int CMD_UNKNOWN_ADDRESS = 0x44;

        }

        int base_address ;
        int ecg_base_address;
        int cmd_base_address;
        string Port;
        bool TX_status = true;
        bool save = false;
        int ecg_pulse = 0;
        int ecg_resp =0;
       string  ecg_Status = "";

        public Form1()
        {
            InitializeComponent();
            this.pictureBox_Graph.Image = new Bitmap(this.pictureBox_Graph.Width, this.pictureBox_Graph.Height);

            bmp = new Bitmap(pictureBox1.Width, pictureBox1.Height);
            
            pictureBox1.Image = bmp;
            g = Graphics.FromImage(bmp);
            g.Clear(Color.White);
            pictureBox1.Invalidate();
            
            textBox_ECG_base_address.Text = ecg_base_address.ToString("X2");
            textBox_base_address.Text = base_address.ToString("X2");
            textBox_CMD_base_address.Text = cmd_base_address.ToString("X2");
            list_comport();
            bool savePort = false;
            Port = Properties.Settings.Default.COMPORT;
            
            textBox_ECG_base_address.Text = Properties.Settings.Default.ECG_Base_address;
            textBox_base_address.Text = Properties.Settings.Default.Base_address;
            textBox_CMD_base_address.Text = Properties.Settings.Default.CMD_Base_address;
            foreach (ToolStripMenuItem item in portToolStripMenuItem.DropDownItems)
            {
                if (item.Text == Port)
                {
                    //item.Checked = true;
                    item.PerformClick();
                    savePort = true;
                }
            }
            if (savePort == false)
            {
                string[] ports = SerialPort.GetPortNames();
                Splashscreen Splash = new Splashscreen(this, ports);
                Splash.UpdatePort += splash_UpdatePort;

                Splash.Show(this);
            }

        }
        
        private void Form1_Load(object sender, EventArgs e)
        {

        }
        
        void splash_UpdatePort(object sender, Splashscreen.TextEventArgs e)
        {
            string port = e.Text;
            foreach (ToolStripMenuItem item in portToolStripMenuItem.DropDownItems)
            {
                if (item.Text == port)
                {
                    item.PerformClick();
                }
            }
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            //if (Properties.Settings.Default.COMPORT != Port)
            {
                Properties.Settings.Default.COMPORT = Port;
                
            }
           //if (Properties.Settings.Default.ECG_Base_address != textBox_ECG_base_address.Text)
            {
                Properties.Settings.Default.ECG_Base_address = textBox_ECG_base_address.Text;
            }
           // if (Properties.Settings.Default.Base_address != textBox_base_address.Text)
            {
                Properties.Settings.Default.Base_address = textBox_base_address.Text;
            }
           // if (Properties.Settings.Default.CMD_Base_address != textBox_CMD_base_address.Text)
            {
                Properties.Settings.Default.CMD_Base_address = textBox_CMD_base_address.Text;
            }
            Properties.Settings.Default.Save();

            if (serialPort1.IsOpen)
                serialPort1.DiscardInBuffer();
                serialPort1.Close();
            
            Environment.Exit(0);
        }
        // search all Comports 
        private void list_comport()
        {
            {
                // Get a list of serial port names. 
                string[] ports = SerialPort.GetPortNames();

                portToolStripMenuItem.DropDownItems.Clear();
                portToolStripMenuItem.DropDownItems.Add("Update...", null, new EventHandler(updateToolStripMenuItem_Click));
                // Display each port name to the console. 
                foreach (string port in ports)
                {

                    portToolStripMenuItem.DropDownItems.Add(port, null, new EventHandler(port_Click));
                }
            }
        }

        private void updateToolStripMenuItem_Click(object sender, EventArgs e)
        {
            list_comport();
            foreach (ToolStripMenuItem item in portToolStripMenuItem.DropDownItems)
            {
                if (item.Text == Port)
                {
                    item.Checked = true;
                }
            }
        }

        // open Comport
        public void port_Click(object sender, EventArgs e)
        {
            if (serialPort1.IsOpen)
                serialPort1.DiscardInBuffer();
            serialPort1.Close();
           
            

            if (((ToolStripMenuItem)sender).Checked == false)
            {
                foreach (ToolStripMenuItem item in portToolStripMenuItem.DropDownItems)
                {
                    item.Checked = false;
                }
                 Port = sender.ToString();
                
                serialPort1.PortName = Port;
                try
                {
                    serialPort1.Open();

                }
                catch
                {
                    MessageBox.Show("Serial port " + serialPort1.PortName +
                            " cannot be opened!", "RS232 tester",
                            MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
                if (serialPort1.IsOpen)
                {
                    foreach (ToolStripMenuItem item in portToolStripMenuItem.DropDownItems)
                    {
                        if (item.Text == Port)
                        {
                            item.Checked = true;
                        }
                    }
                    
                }
            }
            else
            {
                foreach (ToolStripMenuItem item in portToolStripMenuItem.DropDownItems)
                {
                    if (item.Text == Port)
                    {
                        item.Checked = false;
                    }
                }
                
            }


            if (serialPort1.IsOpen)
            {
    
            }

        }
        
        byte[] table = {0x00, 0x5e, 0xbc, 0xe2, 0x61, 0x3f, 0xdd, 0x83,
                        0xc2, 0x9c, 0x7e, 0x20, 0xa3, 0xfd, 0x1f, 0x41,
                        0x9d, 0xc3, 0x21, 0x7f, 0xfc, 0xa2, 0x40, 0x1e,
                        0x5f, 0x01, 0xe3, 0xbd, 0x3e, 0x60, 0x82, 0xdc,
                        0x23, 0x7d, 0x9f, 0xc1, 0x42, 0x1c, 0xfe, 0xa0,
                        0xe1, 0xbf, 0x5d, 0x03, 0x80, 0xde, 0x3c, 0x62,
                        0xbe, 0xe0, 0x02, 0x5c, 0xdf, 0x81, 0x63, 0x3d,
                        0x7c, 0x22, 0xc0, 0x9e, 0x1d, 0x43, 0xa1, 0xff,
                        0x46, 0x18, 0xfa, 0xa4, 0x27, 0x79, 0x9b, 0xc5,
                        0x84, 0xda, 0x38, 0x66, 0xe5, 0xbb, 0x59, 0x07,
                        0xdb, 0x85, 0x67, 0x39, 0xba, 0xe4, 0x06, 0x58,
                        0x19, 0x47, 0xa5, 0xfb, 0x78, 0x26, 0xc4, 0x9a,
                        0x65, 0x3b, 0xd9, 0x87, 0x04, 0x5a, 0xb8, 0xe6,
                        0xa7, 0xf9, 0x1b, 0x45, 0xc6, 0x98, 0x7a, 0x24,
                        0xf8, 0xa6, 0x44, 0x1a, 0x99, 0xc7, 0x25, 0x7b,
                        0x3a, 0x64, 0x86, 0xd8, 0x5b, 0x05, 0xe7, 0xb9,
                        0x8c, 0xd2, 0x30, 0x6e, 0xed, 0xb3, 0x51, 0x0f,
                        0x4e, 0x10, 0xf2, 0xac, 0x2f, 0x71, 0x93, 0xcd,
                        0x11, 0x4f, 0xad, 0xf3, 0x70, 0x2e, 0xcc, 0x92,
                        0xd3, 0x8d, 0x6f, 0x31, 0xb2, 0xec, 0x0e, 0x50,
                        0xaf, 0xf1, 0x13, 0x4d, 0xce, 0x90, 0x72, 0x2c,
                        0x6d, 0x33, 0xd1, 0x8f, 0x0c, 0x52, 0xb0, 0xee,
                        0x32, 0x6c, 0x8e, 0xd0, 0x53, 0x0d, 0xef, 0xb1,
                        0xf0, 0xae, 0x4c, 0x12, 0x91, 0xcf, 0x2d, 0x73,
                        0xca, 0x94, 0x76, 0x28, 0xab, 0xf5, 0x17, 0x49,
                        0x08, 0x56, 0xb4, 0xea, 0x69, 0x37, 0xd5, 0x8b,
                        0x57, 0x09, 0xeb, 0xb5, 0x36, 0x68, 0x8a, 0xd4,
                        0x95, 0xcb, 0x29, 0x77, 0xf4, 0xaa, 0x48, 0x16,
                        0xe9, 0xb7, 0x55, 0x0b, 0x88, 0xd6, 0x34, 0x6a,
                        0x2b, 0x75, 0x97, 0xc9, 0x4a, 0x14, 0xf6, 0xa8,
                        0x74, 0x2a, 0xc8, 0x96, 0x15, 0x4b, 0xa9, 0xf7,
                        0xb6, 0xe8, 0x0a, 0x54, 0xd7, 0x89, 0x6b, 0x35,
                        };

        public byte Checksum(params byte[] val)
        {
            if (val == null)
                throw new ArgumentNullException("val");

            byte c = 0x00;

            foreach (byte b in val)
            {
                c = table[c ^ b];
            }

            return c;
        }
       
        byte[] buffer ={ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
        // parse serial Data
        private void serialPort1_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            while (serialPort1.BytesToRead > 0)
            {
                byte buf;
                buf = (byte)serialPort1.ReadByte();

                if (buf == 0x02)
                {
                   
                    {
                        byte[] tempbuffer ;
                        buffer[0] = buf;
                        if (serialPort1.IsOpen)
                        {
                            buffer[1] = (byte)serialPort1.ReadByte();
                        }

                        if ((buffer[1]&0xF0) == 0xA0)
                        {
                            
                            int temp = ((buffer[1] & 0x0F)+6 );

                            if (temp <= 14)
                            {
                                for (int i = 2; i < temp; i++)
                                {
                                    if (serialPort1.IsOpen)
                                    {
                                        buffer[i] = Convert.ToByte(serialPort1.ReadByte());
                                    }
                                }
                                 
                                tempbuffer = new byte[temp];
                                Buffer.BlockCopy(buffer, 0, tempbuffer, 0, temp);
                               

                                this.BeginInvoke(new nextPackageDeleg(nextPackage), new object[] { tempbuffer });
                               
                            }
                        }
                       
                    }
                }
            }
        }
        
        bool ECG_ch_status_receive = false;
        
        void nextPackage(byte[] Package)
        {
            int cnt;
            
            //int adress = (Package[2]+((Package[3]&0x01)<<8));
            int adress = (Package[2] + (Package[3] << 8));
            if (adress < base_address)
            {
                adress = adress - ecg_base_address;
                //*********************** ECG ********************************************
                if (adress == Constants.ECG_WAVE_ADDRESS)
                {
                    cnt = Convert.ToInt32(textBox_ECG_wave_CNT.Text);
                    cnt++;
                    textBox_ECG_wave_CNT.Text = cnt.ToString();
                    int lenght = (Package[1] & 0x0F);
                    string s = "";
                    for (int i = 0; i < lenght; i++)
                    {
                        s = s + Package[i + 4].ToString("X2") + " ";
                    }
                    textBox_ECG_wave.Text = s;
                    textBox_ECG_wave_CRC.Text = Package[lenght + 4].ToString("X2");
                    byte[] buf = new byte[lenght + 4];
                    Buffer.BlockCopy(Package, 0, buf, 0, lenght + 4);
                    if (Package[lenght + 4] != Checksum(buf))
                    {
                        cnt = Convert.ToInt32(textBox_ECG_wave_ERR.Text);
                        cnt++;
                        textBox_ECG_wave_ERR.Text = cnt.ToString();
                    }
                    ECG_nextLimb(Package);
                }

                else if (adress == Constants.ECG_NUM_ADDRESS)
                {
                    cnt = Convert.ToInt32(textBox_ECG_num_CNT.Text);
                    cnt++;
                    textBox_ECG_num_CNT.Text = cnt.ToString();
                    int lenght = (Package[1] & 0x0F);
                    textBox_ECG_num.Text = Package[4].ToString() + " " + Package[5].ToString();
                    ecg_pulse = Package[4];
                    ecg_resp = Package[5];
                    textBox_ECG_num_CRC.Text = Package[6].ToString("X2");
                    byte[] buf = new byte[lenght + 4];
                    Buffer.BlockCopy(Package, 0, buf, 0, lenght + 4);
                    if (Package[lenght + 4] != Checksum(buf))
                    {
                        cnt = Convert.ToInt32(textBox_ECG_num_ERR.Text);
                        cnt++;
                        textBox_ECG_num_ERR.Text = cnt.ToString();
                    }

                }
                else if (adress == Constants.ECG_STAT_ADDRESS)
                {
                    cnt = Convert.ToInt32(textBox_ECG_stat_CNT.Text);
                    cnt++;
                    textBox_ECG_stat_CNT.Text = cnt.ToString();
                    int lenght = (Package[1] & 0x0F);
                    textBox_ECG_stat.Text = Package[4].ToString("X2") + " " + Package[5].ToString("X2") + " "
                                             + Package[6].ToString("X2") + " " + Package[7].ToString("X2");
                    ecg_Status = textBox_ECG_stat.Text.Replace(" ", "\t");
                    textBox_ECG_stat_CRC.Text = Package[8].ToString("X2");
                    byte[] buf = new byte[lenght + 4];
                    Buffer.BlockCopy(Package, 0, buf, 0, lenght + 4);
                    if (Package[lenght + 4] != Checksum(buf))
                    {
                        cnt = Convert.ToInt32(textBox_ECG_stat_ERR.Text);
                        cnt++;
                        textBox_ECG_stat_ERR.Text = cnt.ToString();
                    }
                    if (ECG_ch_status_receive == false)
                    {
                        ECG_ch_status_receive = true;
                        int tmp_cnt = 0;
                        if ((Package[5] & 0x01) != 0)
                        {
                            checkI.Checked = true;
                            tmp_cnt++;
                        }
                        if ((Package[5] & 0x02) != 0)
                        {
                            checkII.Checked = true;
                            tmp_cnt++;
                        }
                        if ((Package[5] & 0x04) != 0)
                        {
                            checkIII.Checked = true;
                            tmp_cnt++;
                        }
                        if ((Package[5] & 0x08) != 0)
                        {
                            checkaVR.Checked = true;
                            tmp_cnt++;
                        }
                        if ((Package[5] & 0x10) != 0)
                        {
                            checkaVL.Checked = true;
                            tmp_cnt++;
                        }
                        if ((Package[5] & 0x20) != 0)
                        {
                            checkaVF.Checked = true;
                            tmp_cnt++;
                        }
                        if ((Package[5] & 0x40) != 0)
                        {
                            checkC.Checked = true;
                            tmp_cnt++;
                        }
                        if ((Package[4] & 0x40) != 0)
                        {
                            checkR.Checked = true;
                            tmp_cnt++;
                        }
                        cnt_Channel = tmp_cnt;
                    }
                }
                //*********************** IBP1 ********************************************
                else if (adress == Constants.IBP1_WAVE_ADDRESS)
                {
                    cnt = Convert.ToInt32(textBox_IBP1_Wave_cnt.Text);
                    cnt++;
                    textBox_IBP1_Wave_cnt.Text = cnt.ToString();
                    int lenght = (Package[1] & 0x0F);
                    textBox_IBP1_wave.Text = ((int)(Package[5] + (Package[4] & (0x03) << 7))).ToString() + " " + ((int)(Package[6] + (Package[4] & (0x0C) << 5))).ToString();
                    textBox_IBP1_wave_crc.Text = Package[7].ToString("X2");
                    byte[] buf = new byte[lenght + 4];
                    Buffer.BlockCopy(Package, 0, buf, 0, lenght + 4);
                    if (Package[lenght + 4] != Checksum(buf))
                    {
                        cnt = Convert.ToInt32(textBox_IBP1_wave_err.Text);
                        cnt++;
                        textBox_IBP1_wave_err.Text = cnt.ToString();
                    }
                }
                else if (adress == Constants.IBP1_INFO1_ADDRESS)
                {
                    cnt = Convert.ToInt32(textBox_IBP1_info1_cnt.Text);
                    cnt++;
                    textBox_IBP1_info1_cnt.Text = cnt.ToString();
                    int lenght = (Package[1] & 0x0F);
                    textBox_IBP1_info1.Text = ((int)(Package[5] + (Package[4] & (0x30) << 3))).ToString() + " " + ((int)(Package[6] + (Package[4] & (0x0C) << 5))).ToString() + " " +((int)(Package[7] + (Package[4] & (0x03) << 7))).ToString();
                    textBox_IBP1_info1_crc.Text = Package[8].ToString("X2");
                    byte[] buf = new byte[lenght + 4];
                    Buffer.BlockCopy(Package, 0, buf, 0, lenght + 4);
                    if (Package[lenght + 4] != Checksum(buf))
                    {
                        cnt = Convert.ToInt32(textBox_IBP1_info1_err.Text);
                        cnt++;
                        textBox_IBP1_info1_err.Text = cnt.ToString();
                    }
                }
                else if (adress == Constants.IBP1_INFO2_ADDRESS)
                {
                    cnt = Convert.ToInt32(textBox_IBP1_info2_cnt.Text);
                    cnt++;
                    textBox_IBP1_info2_cnt.Text = cnt.ToString();
                    int lenght = (Package[1] & 0x0F);
                    textBox_IBP1_info2.Text = ((int)(Package[5] + (Package[4] & (0x30) << 3))).ToString() + " " + ((int)(Package[6] + (Package[4] & (0x0C) << 5))).ToString() + " " + ((int)(Package[7] + (Package[4] & (0x03) << 7))).ToString() + " " + ((int)(Package[8] + (Package[4] & (0x40) << 1))).ToString();
                    textBox_IBP1_info2_crc.Text = Package[9].ToString("X2");
                    byte[] buf = new byte[lenght + 4];
                    Buffer.BlockCopy(Package, 0, buf, 0, lenght + 4);
                    if (Package[lenght + 4] != Checksum(buf))
                    {
                        cnt = Convert.ToInt32(textBox_IBP1_info2_err.Text);
                        cnt++;
                        textBox_IBP1_info2_err.Text = cnt.ToString();
                    }
                }
                else if (adress == Constants.IBP1_STAT_ADDRESS)
                {
                    cnt = Convert.ToInt32(textBox_IBP1_Stat_cnt.Text);
                    cnt++;
                    textBox_IBP1_Stat_cnt.Text = cnt.ToString();
                    int lenght = (Package[1] & 0x0F);
                    textBox_IBP1_stat.Text = ((Package[4] & (0x0F) )).ToString("X2") + " " + (Package[5] & (0x0F)).ToString("X2");
                    textBox_IBP1_stat_crc.Text = Package[7].ToString("X2");
                    byte[] buf = new byte[lenght + 4];
                    Buffer.BlockCopy(Package, 0, buf, 0, lenght + 4);
                    if (Package[lenght + 4] != Checksum(buf))
                    {
                        cnt = Convert.ToInt32(textBox_IBP1_stat_err.Text);
                        cnt++;
                        textBox_IBP1_stat_err.Text = cnt.ToString();
                    }
                }
                //*********************** IBP2 ********************************************
                else if (adress == Constants.IBP2_WAVE_ADDRESS)
                {

                }
                else if (adress == Constants.IBP2_INFO1_ADDRESS)
                {

                }
                else if (adress == Constants.IBP2_INFO2_ADDRESS)
                {

                }
                else if (adress == Constants.IBP2_STAT_ADDRESS)
                {

                }


            }
            else
            {
                adress = adress - base_address;
                //*********************** SPO2 ********************************************
                if (adress == Constants.SPO_WAVE_ADDRESS)
                {
                    cnt = Convert.ToInt32(textBox_SPO_wave_CNT.Text);
                    cnt++;
                    textBox_SPO_wave_CNT.Text = cnt.ToString();
                    int lenght = (Package[1] & 0x0F);
                    textBox_SPO_wave.Text = Package[4].ToString("X2");
                    textBox_SPO_wave_CRC.Text = Package[5].ToString("X2");
                    byte[] buf = new byte[lenght + 4];
                    Buffer.BlockCopy(Package, 0, buf, 0, lenght + 4);
                    if (Package[lenght + 4] != Checksum(buf))
                    {
                        cnt = Convert.ToInt32(textBox_SPO_wave_ERR.Text);
                        cnt++;
                        textBox_SPO_wave_ERR.Text = cnt.ToString();
                    }
                    SPO_Display(Package[4]);
                }

                else if (adress == Constants.SPO_NUM_ADDRESS)
                {
                    cnt = Convert.ToInt32(textBox_SPO_num_CNT.Text);
                    cnt++;
                    textBox_SPO_num_CNT.Text = cnt.ToString();
                    int lenght = (Package[1] & 0x0F);
                    textBox_SPO_num.Text = Package[4].ToString() + " " + Package[5].ToString();
                    textBox_SPO_num_CRC.Text = Package[6].ToString("X2");
                    byte[] buf = new byte[lenght + 4];
                    Buffer.BlockCopy(Package, 0, buf, 0, lenght + 4);
                    if (Package[lenght + 4] != Checksum(buf))
                    {
                        cnt = Convert.ToInt32(textBox_SPO_num_ERR.Text);
                        cnt++;
                        textBox_SPO_num_ERR.Text = cnt.ToString();
                    }
                }

                else if (adress == Constants.SPO_STAT_ADDRESS)
                {
                    cnt = Convert.ToInt32(textBox_SPO_stat_CNT.Text);
                    cnt++;
                    textBox_SPO_stat_CNT.Text = cnt.ToString();
                    int lenght = (Package[1] & 0x0F);
                    textBox_SPO_stat.Text = Package[4].ToString("X2") + " " + Package[5].ToString("X2") + " " + Package[6].ToString("X2");
                    textBox_SPO_stat_CRC.Text = Package[5].ToString("X2");
                    byte[] buf = new byte[lenght + 4];
                    Buffer.BlockCopy(Package, 0, buf, 0, lenght + 4);
                    if (Package[lenght + 4] != Checksum(buf))
                    {
                        cnt = Convert.ToInt32(textBox_SPO_stat_ERR.Text);
                        cnt++;
                        textBox_SPO_stat_ERR.Text = cnt.ToString();
                    }
                    SPO_Status = Package[4];
                }
                //*********************** NIBP ********************************************
                else if (adress == Constants.NIBP_CUFF_ADDRESS)
                {
                    cnt = Convert.ToInt32(textBox_NIBP_press_CNT.Text);
                    cnt++;
                    textBox_NIBP_press_CNT.Text = cnt.ToString();
                    int lenght = (Package[1] & 0x0F);
                    textBox_NIBP_press.Text = ((int)(Package[4] + (Package[5] << 8))).ToString();
                    textBox_NIBP_press_CRC.Text = Package[6].ToString("X2");
                    byte[] buf = new byte[lenght + 4];
                    Buffer.BlockCopy(Package, 0, buf, 0, lenght + 4);
                    if (Package[lenght + 4] != Checksum(buf))
                    {
                        cnt = Convert.ToInt32(textBox_NIBP_press_ERR.Text);
                        cnt++;
                        textBox_NIBP_press_ERR.Text = cnt.ToString();
                    }
                }

                else if (adress == Constants.NIBP_NUM_ADDRESS)
                {
                    cnt = Convert.ToInt32(textBox_NIBP_num_CNT.Text);
                    cnt++;
                    textBox_NIBP_num_CNT.Text = cnt.ToString();
                    int lenght = (Package[1] & 0x0F);
                    textBox_NIBP_num.Text = ((int)(Package[4] + (Package[5] << 8))).ToString() + " " +
                                            ((int)(Package[6] + (Package[7] << 8))).ToString() + " " +
                                            ((int)(Package[8] + (Package[9] << 8))).ToString() + " " +
                                            Package[10].ToString();
                    textBox_NIBP_num_CRC.Text = Package[11].ToString("X2");
                    byte[] buf = new byte[lenght + 4];
                    Buffer.BlockCopy(Package, 0, buf, 0, lenght + 4);
                    if (Package[lenght + 4] != Checksum(buf))
                    {
                        cnt = Convert.ToInt32(textBox_NIBP_num_ERR.Text);
                        cnt++;
                        textBox_NIBP_num_ERR.Text = cnt.ToString();
                    }
                }

                else if (adress == Constants.NIBP_STAT_ADDRESS)
                {
                    cnt = Convert.ToInt32(textBox_NIBP_stat_CNT.Text);
                    cnt++;
                    textBox_NIBP_stat_CNT.Text = cnt.ToString();
                    int lenght = (Package[1] & 0x0F);
                    textBox_NIBP_stat.Text = Package[4].ToString("X2") + " " + Package[5].ToString("X2") + " "
                                             + Package[6].ToString("X2") + " " + Package[7].ToString("X2");
                    textBox_NIBP_stat_CRC.Text = Package[8].ToString("X2");
                    byte[] buf = new byte[lenght + 4];
                    Buffer.BlockCopy(Package, 0, buf, 0, lenght + 4);
                    if (Package[lenght + 4] != Checksum(buf))
                    {
                        cnt = Convert.ToInt32(textBox_NIBP_stat_ERR.Text);
                        cnt++;
                        textBox_NIBP_stat_ERR.Text = cnt.ToString();
                    }
                }

                else if (adress == Constants.NIBP_TIMER_ADDRESS)
                {
                    cnt = Convert.ToInt32(textBox_NIBP_timer_CNT.Text);
                    cnt++;
                    textBox_NIBP_timer_CNT.Text = cnt.ToString();
                    int lenght = (Package[1] & 0x0F);
                    textBox_NIBP_timer.Text = ((int)(Package[4] + (Package[5] << 8))).ToString() + " " +
                                            ((int)(Package[6] + (Package[7] << 8))).ToString();
                    textBox_NIBP_timer_CRC.Text = Package[8].ToString("X2");
                    byte[] buf = new byte[lenght + 4];
                    Buffer.BlockCopy(Package, 0, buf, 0, lenght + 4);
                    if (Package[lenght + 4] != Checksum(buf))
                    {
                        cnt = Convert.ToInt32(textBox_NIBP_timer_ERR.Text);
                        cnt++;
                        textBox_NIBP_timer_ERR.Text = cnt.ToString();
                    }
                }

                



                //***************************** CO2 ********************************************+
                else if (adress == Constants.CO2_VALUE_ADDRESS)
                {
                    cnt = Convert.ToInt32(textBox_CO2_value_cnt.Text);
                    cnt++;
                    textBox_CO2_value_cnt.Text = cnt.ToString();
                    int lenght = (Package[1] & 0x0F);
                    textBox_CO2_Value.Text = ((int)(Package[5] + (Package[4] << 8))).ToString();
                    textBox_CO2_value_crc.Text = Package[6].ToString("X2");
                    byte[] buf = new byte[lenght + 4];
                    Buffer.BlockCopy(Package, 0, buf, 0, lenght + 4);
                    if (Package[lenght + 4] != Checksum(buf))
                    {
                        cnt = Convert.ToInt32(textBox_CO2_value_err.Text);
                        cnt++;
                        textBox_CO2_value_err.Text = cnt.ToString();
                    }
                }

                else if (adress == Constants.CO2_TEMP_ADDRESS)
                {
                    cnt = Convert.ToInt32(textBox_CO2_temp_cnt.Text);
                    cnt++;
                    textBox_CO2_temp_cnt.Text = cnt.ToString();
                    int lenght = (Package[1] & 0x0F);
                    textBox_CO2_temp.Text = ((int)(Package[5] + (Package[4] << 8))).ToString();
                    textBox_CO2_Temp_crc.Text = Package[6].ToString("X2");
                    byte[] buf = new byte[lenght + 4];
                    Buffer.BlockCopy(Package, 0, buf, 0, lenght + 4);
                    if (Package[lenght + 4] != Checksum(buf))
                    {
                        cnt = Convert.ToInt32(textBox_CO2_Temp_err.Text);
                        cnt++;
                        textBox_CO2_Temp_err.Text = cnt.ToString();
                    }
                }

                else if (adress == Constants.CO2_AMB_ADDRESS)
                {

                    cnt = Convert.ToInt32(textBox_CO2_Amb_cnt.Text);
                    cnt++;
                    textBox_CO2_Amb_cnt.Text = cnt.ToString();
                    int lenght = (Package[1] & 0x0F);
                    textBox_CO2_amb.Text = ((int)(Package[5] + (Package[4] << 8))).ToString();
                    textBox_CO2_Amb_crc.Text = Package[6].ToString("X2");
                    byte[] buf = new byte[lenght + 4];
                    Buffer.BlockCopy(Package, 0, buf, 0, lenght + 4);
                    if (Package[lenght + 4] != Checksum(buf))
                    {
                        cnt = Convert.ToInt32(textBox_CO2_Amb_err.Text);
                        cnt++;
                        textBox_CO2_Amb_err.Text = cnt.ToString();
                    }
                }

                else if (adress == Constants.CO2_WAVE_ADDRESS)
                {
                    cnt = Convert.ToInt32(textBox_CO2_wave_cnt.Text);
                    cnt++;
                    textBox_CO2_wave_cnt.Text = cnt.ToString();
                    int lenght = (Package[1] & 0x0F);
                    textBox_CO2_Wave.Text = ((int)(Package[5] + (Package[4] << 8))).ToString();
                    textBox_CO2_wave_crc.Text = Package[6].ToString("X2");
                    byte[] buf = new byte[lenght + 4];
                    Buffer.BlockCopy(Package, 0, buf, 0, lenght + 4);
                    if (Package[lenght + 4] != Checksum(buf))
                    {
                        cnt = Convert.ToInt32(textBox_CO2_wave_err.Text);
                        cnt++;
                        textBox_CO2_wave_err.Text = cnt.ToString();
                    }
                }
                else if (adress == Constants.CO2_RESP_ADDRESS)
                {
                    cnt = Convert.ToInt32(textBox_CO2_resp_cnt.Text);
                    cnt++;
                    textBox_CO2_resp_cnt.Text = cnt.ToString();
                    int lenght = (Package[1] & 0x0F);
                    textBox_CO2_resp.Text = ((int)(Package[4])).ToString();
                    textBox_CO2_resp_crc.Text = Package[6].ToString("X2");
                    byte[] buf = new byte[lenght + 4];
                    Buffer.BlockCopy(Package, 0, buf, 0, lenght + 4);
                    if (Package[lenght + 4] != Checksum(buf))
                    {
                        cnt = Convert.ToInt32(textBox_CO2_resp_err.Text);
                        cnt++;
                        textBox_CO2_resp_err.Text = cnt.ToString();
                    }
                }
                else if (adress == Constants.CO2_FLOW_ADDRESS)
                {
                    cnt = Convert.ToInt32(textBox_CO2_flow_cnt.Text);
                    cnt++;
                    textBox_CO2_flow_cnt.Text = cnt.ToString();
                    int lenght = (Package[1] & 0x0F);
                    textBox_CO2_flow.Text = ((int)(Package[5] + (Package[4] << 8))).ToString();
                    textBox_CO2_flow_crc.Text = Package[6].ToString("X2");
                    byte[] buf = new byte[lenght + 4];
                    Buffer.BlockCopy(Package, 0, buf, 0, lenght + 4);
                    if (Package[lenght + 4] != Checksum(buf))
                    {
                        cnt = Convert.ToInt32(textBox_CO2_flow_err.Text);
                        cnt++;
                        textBox_CO2_flow_err.Text = cnt.ToString();
                    }

                }
                else if (adress == Constants.CO2_INCO2_ADDRESS)
                {
                    cnt = Convert.ToInt32(textBox_CO2_InCO2_cnt.Text);
                    cnt++;
                    textBox_CO2_InCO2_cnt.Text = cnt.ToString();
                    int lenght = (Package[1] & 0x0F);
                    textBox_CO2_inCO2.Text = ((int)(Package[5] + (Package[4] << 8))).ToString();
                    textBox_CO2_InCO2_crc.Text = Package[6].ToString("X2");
                    byte[] buf = new byte[lenght + 4];
                    Buffer.BlockCopy(Package, 0, buf, 0, lenght + 4);
                    if (Package[lenght + 4] != Checksum(buf))
                    {
                        cnt = Convert.ToInt32(textBox_CO2_InCO2_err.Text);
                        cnt++;
                        textBox_CO2_InCO2_err.Text = cnt.ToString();
                    }
                }

                else if (adress == Constants.CO2_FIO2_ADDRESS)
                {
                    cnt = Convert.ToInt32(textBox_CO2_FiO2_cnt.Text);
                    cnt++;
                    textBox_CO2_FiO2_cnt.Text = cnt.ToString();
                    int lenght = (Package[1] & 0x0F);
                    textBox_CO2_FiO2.Text = ((int)(Package[5] + (Package[4] << 8))).ToString();
                    textBox_CO2_FiO2_crc.Text = Package[6].ToString("X2");
                    byte[] buf = new byte[lenght + 4];
                    Buffer.BlockCopy(Package, 0, buf, 0, lenght + 4);
                    if (Package[lenght + 4] != Checksum(buf))
                    {
                        cnt = Convert.ToInt32(textBox_CO2_FiO2_err.Text);
                        cnt++;
                        textBox_CO2_FiO2_err.Text = cnt.ToString();
                    }
                }
                else if (adress == Constants.CO2_ERR_ADDRESS)
                {
                    cnt = Convert.ToInt32(textBox_CO2_err_cnt.Text);
                    cnt++;
                    textBox_CO2_err_cnt.Text = cnt.ToString();
                    int lenght = (Package[1] & 0x0F);
                    textBox_CO2_err.Text = ((Package[4])).ToString("X2");
                    textBox_CO2_err_crc.Text = Package[6].ToString("X2");
                    byte[] buf = new byte[lenght + 4];
                    Buffer.BlockCopy(Package, 0, buf, 0, lenght + 4);
                    if (Package[lenght + 4] != Checksum(buf))
                    {
                        cnt = Convert.ToInt32(textBox_CO2_err_err.Text);
                        cnt++;
                        textBox_CO2_err_err.Text = cnt.ToString();
                    }
                }

                else if (adress == Constants.CO2_INFO_ADDRESS)
                {
                    cnt = Convert.ToInt32(textBox_CO2_info_cnt.Text);
                    cnt++;
                    textBox_CO2_info_cnt.Text = cnt.ToString();
                    int lenght = (Package[1] & 0x0F);
                    textBox_CO2_info.Text = ((Package[4])).ToString("X2");
                    textBox_CO2_info_crc.Text = Package[6].ToString("X2");
                    byte[] buf = new byte[lenght + 4];
                    Buffer.BlockCopy(Package, 0, buf, 0, lenght + 4);
                    if (Package[lenght + 4] != Checksum(buf))
                    {
                        cnt = Convert.ToInt32(textBox_CO2_info_err.Text);
                        cnt++;
                        textBox_CO2_info_err.Text = cnt.ToString();
                    }
                }

                //*********************** TEMP ******************************************
                else if (adress == Constants.TEMP_NUM_ADDRESS)
                {
                    cnt = Convert.ToInt32(textBox_TEMP_num_CNT.Text);
                    cnt++;
                    textBox_TEMP_num_CNT.Text = cnt.ToString();
                    int lenght = (Package[1] & 0x0F);
                    textBox_TEMP_num.Text = ((double)((double)(Package[4] + (Package[5] << 8)) / 10.0)).ToString("0.0") + " " +
                                            ((double)((double)(Package[6] + (Package[7] << 8)) / 10.0)).ToString("0.0") + " " +
                                            ((double)((double)(Package[8] + (Package[9] << 8)) / 10.0)).ToString("0.0");

                    textBox_TEMP_num_CRC.Text = Package[10].ToString("X2");
                    byte[] buf = new byte[lenght + 4];
                    Buffer.BlockCopy(Package, 0, buf, 0, lenght + 4);
                    if (Package[lenght + 4] != Checksum(buf))
                    {
                        cnt = Convert.ToInt32(textBox_TEMP_num_ERR.Text);
                        cnt++;
                        textBox_TEMP_num_ERR.Text = cnt.ToString();
                    }
                }

                else if (adress == Constants.TEMP_STAT_ADDRESS)
                {
                    cnt = Convert.ToInt32(textBox_TEMP_stat_CNT.Text);
                    cnt++;
                    textBox_TEMP_stat_CNT.Text = cnt.ToString();
                    int lenght = (Package[1] & 0x0F);
                    textBox_TEMP_stat.Text = Package[4].ToString("X2") + " " + Package[5].ToString("X2") + " "
                                             + Package[6].ToString("X2");

                    textBox_TEMP_stat_CRC.Text = Package[7].ToString("X2");
                    byte[] buf = new byte[lenght + 4];
                    Buffer.BlockCopy(Package, 0, buf, 0, lenght + 4);
                    if (Package[lenght + 4] != Checksum(buf))
                    {
                        cnt = Convert.ToInt32(textBox_TEMP_stat_ERR.Text);
                        cnt++;
                        textBox_TEMP_stat_ERR.Text = cnt.ToString();
                    }
                }
                //*********************** General ************************************************
                else if (adress == Constants.MULTI_STAT_ADDRESS)
                {
                    cnt = Convert.ToInt32(textBox_MULTI_stat_CNT.Text);
                    cnt++;
                    textBox_MULTI_stat_CNT.Text = cnt.ToString();
                    int lenght = (Package[1] & 0x0F);
                    textBox_MULTI_stat.Text = ((int)(Package[4] + (Package[5] << 8))).ToString() + " " +
                                            ((int)(Package[6] + (Package[7] << 8))).ToString() + " " +
                                            ((int)(Package[8] + (Package[9] << 8))).ToString();

                    textBox_MULTI_stat_CRC.Text = Package[10].ToString("X2");
                    byte[] buf = new byte[lenght + 4];
                    Buffer.BlockCopy(Package, 0, buf, 0, lenght + 4);
                    if (Package[lenght + 4] != Checksum(buf))
                    {
                        cnt = Convert.ToInt32(textBox_MULTI_stat_ERR.Text);
                        cnt++;
                        textBox_MULTI_stat_ERR.Text = cnt.ToString();
                    }
                }
                else if (adress == Constants.MULTI_VERSION_ADDRESS)
                {
                    cnt = Convert.ToInt32(textBox_MULTI_version_CNT.Text);
                    cnt++;
                    textBox_MULTI_version_CNT.Text = cnt.ToString();
                    int lenght = (Package[1] & 0x0F);
                    textBox_MULTI_version.Text = Package[4].ToString("X2") + " " + Package[5].ToString("X2") + " "
                                             + Package[6].ToString("X2") + " " + Package[7].ToString("X2");

                    textBox_MULTI_version_CRC.Text = Package[8].ToString("X2");
                    byte[] buf = new byte[lenght + 4];
                    Buffer.BlockCopy(Package, 0, buf, 0, lenght + 4);
                    if (Package[lenght + 4] != Checksum(buf))
                    {
                        cnt = Convert.ToInt32(textBox_MULTI_version_ERR.Text);
                        cnt++;
                        textBox_MULTI_version_ERR.Text = cnt.ToString();
                    }
                }
                else if (adress == Constants.MULTI_NUM_ADDRESS)
                {
                    cnt = Convert.ToInt32(textBox_MULTI_num_CNT.Text);
                    cnt++;
                    textBox_MULTI_num_CNT.Text = cnt.ToString();
                    int lenght = (Package[1] & 0x0F);
                    textBox_MULTI_num.Text = Package[4].ToString("X2") + " " + Package[5].ToString("X2") + " "
                                             + Package[6].ToString("X2") + " " + Package[7].ToString("X2");

                    textBox_MULTI_num_CRC.Text = Package[8].ToString("X2");
                    byte[] buf = new byte[lenght + 4];
                    Buffer.BlockCopy(Package, 0, buf, 0, lenght + 4);
                    if (Package[lenght + 4] != Checksum(buf))
                    {
                        cnt = Convert.ToInt32(textBox_MULTI_num_ERR.Text);
                        cnt++;
                        textBox_MULTI_num_ERR.Text = cnt.ToString();
                    }
                }

                else if (adress == Constants.CMD_ACK_ADDRESS)
                {
                    timer1.Stop();
                    if (Command_Flag == 0x01)
                    {
                        checkBox_ECG_ack.Checked = true;
                    }
                    else if (Command_Flag == 0x02)
                    {
                        checkBox_SPO_ack.Checked = true;
                    }
                    else if (Command_Flag == 0x04)
                    {
                        checkBox_NIBP_ack.Checked = true;
                    }
                    else if (Command_Flag == 0x08)
                    {
                        checkBox_TEMP_ack.Checked = true;
                    }
                    else if (Command_Flag == 0x10)
                    {
                        checkBox_MULTI_ack.Checked = true;
                    }
                    Command_Flag = 0;
                }

                else if (adress == Constants.CMD_ERR_ADDRESS)
                {
                    timer1.Stop();
                    Command_Flag = 0;
                    MessageBox.Show("Command Error Frame");
                }

                else if (adress == Constants.CMD_TIMEOUT_ADDRESS)
                {
                    timer1.Stop();
                    Command_Flag = 0;
                    MessageBox.Show("Command Error TimeOut");
                }

                else if (adress == Constants.CMD_ERR_CRC_ADDRESS)
                {
                    timer1.Stop();
                    Command_Flag = 0;
                    MessageBox.Show("Command Error CRC");
                }

                else if (adress == Constants.CMD_UNKNOWN_ADDRESS)
                {
                    timer1.Stop();
                    Command_Flag = 0;
                    MessageBox.Show("Command Unknown");
                }
                //********************** wrong adress ************************************
                else
                {
                    Console.WriteLine("Wrong Address");
                }
            }
        }

        private void button_NIBP_crc_Click(object sender, EventArgs e)
        {
            byte[] buffer = new byte[7];
            buffer[0] = 0x02;
            buffer[1] = 0xA3;
            int tmpadress = cmd_base_address + Constants.NIBP_CMD_ADDRESS;
            buffer[2] = (byte)(tmpadress & 0x00ff);
            buffer[3] = (byte)((tmpadress & 0xff00) >> 8);
            buffer[4] = Constants.NIBP_IDENT;
            char[]temp = textBox_NIBP_comm.Text.ToCharArray();
            buffer[5] = Convert.ToByte(temp[0]);
            buffer[6] = Convert.ToByte(temp[1]);
            int temp1 = Checksum(buffer);
            textBox_NIBP_comm_CRC.Text = temp1.ToString("X2");
            checkBox_NIBP_ack.Checked = false;

        }
        
        int Command_Flag = 0;

        private void button_NIBP_send_Click(object sender, EventArgs e)
        {
            byte[] buffer = new byte[9];
            buffer[0] = 0x02;
            buffer[1] = 0xA3;
            int tmpadress = cmd_base_address + Constants.NIBP_CMD_ADDRESS;
            buffer[2] = (byte)(tmpadress & 0x00ff);
            buffer[3] = (byte)((tmpadress & 0xff00) >> 8);
            buffer[4] = Constants.NIBP_IDENT;
            char[] temp = textBox_NIBP_comm.Text.ToCharArray();
            buffer[5] = Convert.ToByte(temp[0]);
            buffer[6] = Convert.ToByte(temp[1]);
            if (textBox_NIBP_comm_CRC.Text != "")
            {
                buffer[7] = (byte)Convert.ToInt32(textBox_NIBP_comm_CRC.Text, 16);
                //buffer[7] = Checksum(buffer);
                buffer[8] = 0x03;
                if (Command_Flag == 0)
                {
                    if (serialPort1.IsOpen)
                    {
                        serialPort1.Write(buffer, 0, buffer.Length);
                        Command_Flag = (1 << 2);
                        timer1.Start();
                    }
                }
                else
                {
                    MessageBox.Show("Ein Kommando noch nicht besttigt");
                }
            }
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            timer1.Stop();
            Command_Flag = 0;
            MessageBox.Show("Timeout Kommando wurde nicht besttigt");
        }

        private void textBox_NIBP_comm_TextChanged(object sender, EventArgs e)
        {
            textBox_NIBP_comm_CRC.Text = "";
            checkBox_NIBP_ack.Checked = false;
        }
        
        private void textBox_SPO_comm_TextChanged(object sender, EventArgs e)
        {
            textBox_SPO_comm_CRC.Text = "";
            checkBox_SPO_ack.Checked = false;
        }
        
        private void textBox_ECG_comm_TextChanged(object sender, EventArgs e)
        {
            textBox_ECG_comm_CRC.Text = "";
            checkBox_ECG_ack.Checked = false;
        }
        
        private void textBox_TEMP_comm_TextChanged(object sender, EventArgs e)
        {
            textBox_TEMP_comm_CRC.Text = "";
            checkBox_TEMP_ack.Checked = false;
        }
        
        private void textBox_MULTI_comm_TextChanged(object sender, EventArgs e)
        {
            textBox_MULTI_comm_CRC.Text = "";
            checkBox_MULTI_ack.Checked = false;
        }

        private void textBox_CO2_comm_TextChanged(object sender, EventArgs e)
        {
            textBox_Co2_comm_CRC.Text = "";
            checkBox_CO2_Ack.Checked = false;
        }
        private void textBox_IBP1_comm_TextChanged(object sender, EventArgs e)
        {
            textBox_IBP1_comm_CRC.Text = "";
            checkBox_IBP1_ack.Checked = false;
        }
        private void textBox_IBP2_comm_TextChanged(object sender, EventArgs e)
        {
            textBox_IBP2_comm_CRC.Text = "";
            checkBox_IBP2_ack.Checked = false;
        }

        private void button_ECG_crc_Click(object sender, EventArgs e)
        {
            byte[] buffer = new byte[7];
            buffer[0] = 0x02;
            buffer[1] = 0xA3;
            int tmpadress = cmd_base_address + Constants.ECG_CMD_ADDRESS;
            buffer[2] = (byte)(tmpadress & 0x00ff);
            buffer[3] = (byte)((tmpadress & 0xff00) >> 8);
            buffer[4] = Constants.ECG_IDENT;
            char[] temp = textBox_ECG_comm.Text.ToCharArray();
            buffer[5] = Convert.ToByte(temp[0]);
            buffer[6] = Convert.ToByte(temp[1]);
            int temp1 = Checksum(buffer);
            textBox_ECG_comm_CRC.Text = temp1.ToString("X2");
            checkBox_ECG_ack.Checked = false;
        }

        private void button_ECG_send_Click(object sender, EventArgs e)
        {
            byte[] buffer = new byte[9];
            buffer[0] = 0x02;
            buffer[1] = 0xA3;
            int tmpadress = cmd_base_address + Constants.ECG_CMD_ADDRESS;
            buffer[2] = (byte)(tmpadress & 0x00ff);
            buffer[3] = (byte)((tmpadress & 0xff00) >> 8);
            buffer[4] = Constants.ECG_IDENT;
            char[] temp = textBox_ECG_comm.Text.ToCharArray();
            buffer[5] = Convert.ToByte(temp[0]);
            buffer[6] = Convert.ToByte(temp[1]);
            if (textBox_ECG_comm_CRC.Text != "")
            {

                buffer[7] = (byte)Convert.ToInt32(textBox_ECG_comm_CRC.Text, 16);
                buffer[8] = 0x03;
                if (Command_Flag == 0)
                {
                    if (serialPort1.IsOpen)
                    {
                        serialPort1.Write(buffer, 0, buffer.Length);
                        Command_Flag = (1 << 0);
                        timer1.Start();
                    }
                }
                else
                {
                    MessageBox.Show("Ein Kommando noch nicht besttigt");
                }
            }
        }

        private void button_SPO_crc_Click(object sender, EventArgs e)
        {
            byte[] buffer = new byte[7];
            buffer[0] = 0x02;
            buffer[1] = 0xA3;
            int tmpadress = cmd_base_address + Constants.SPO_CMD_ADDRESS;
            buffer[2] = (byte)(tmpadress & 0x00ff);
            buffer[3] = (byte)((tmpadress & 0xff00) >> 8);
            buffer[4] = Constants.SPO_IDENT;
            char[] temp = textBox_SPO_comm.Text.ToCharArray();
            buffer[5] = Convert.ToByte(temp[0]);
            buffer[6] = Convert.ToByte(temp[1]);
            int temp1 = Checksum(buffer);
            textBox_SPO_comm_CRC.Text = temp1.ToString("X2");
            checkBox_SPO_ack.Checked = false;
        }

        private void button_SPO_send_Click(object sender, EventArgs e)
        {
            byte[] buffer = new byte[9];
            buffer[0] = 0x02;
            buffer[1] = 0xA3;
            int tmpadress = cmd_base_address + Constants.SPO_CMD_ADDRESS;
            buffer[2] = (byte)(tmpadress & 0x00ff);
            buffer[3] = (byte)((tmpadress & 0xff00) >> 8);
            buffer[4] = Constants.SPO_IDENT;
            char[] temp = textBox_SPO_comm.Text.ToCharArray();
            buffer[5] = Convert.ToByte(temp[0]);
            buffer[6] = Convert.ToByte(temp[1]);
            if (textBox_SPO_comm_CRC.Text != "")
            {
                buffer[7] = (byte)Convert.ToInt32(textBox_SPO_comm_CRC.Text, 16);
                buffer[8] = 0x03;
                if (Command_Flag == 0)
                {
                    if (serialPort1.IsOpen)
                    {
                        serialPort1.Write(buffer, 0, buffer.Length);
                        Command_Flag = (1 << 1);
                        timer1.Start();
                    }
                }
                else
                {
                    MessageBox.Show("Ein Kommando noch nicht besttigt");
                }
            }
        }

        private void button_TEMP_crc_Click(object sender, EventArgs e)
        {
            byte[] buffer = new byte[7];
            buffer[0] = 0x02;
            buffer[1] = 0xA3;
            int tmpadress = cmd_base_address + Constants.TEMP_CMD_ADDRESS;
            buffer[2] = (byte)(tmpadress & 0x00ff);
            buffer[3] = (byte)((tmpadress & 0xff00) >> 8);
            buffer[4] = Constants.TEMP_IDENT;
            char[] temp = textBox_TEMP_comm.Text.ToCharArray();
            buffer[5] = Convert.ToByte(temp[0]);
            buffer[6] = Convert.ToByte(temp[1]);
            int temp1 = Checksum(buffer);
            textBox_TEMP_comm_CRC.Text = temp1.ToString("X2");
            checkBox_TEMP_ack.Checked = false;
        }

        private void button_TEMP_send_Click(object sender, EventArgs e)
        {
            byte[] buffer = new byte[9];
            buffer[0] = 0x02;
            buffer[1] = 0xA3;
            int tmpadress = cmd_base_address + Constants.TEMP_CMD_ADDRESS;
            buffer[2] = (byte)(tmpadress & 0x00ff);
            buffer[3] = (byte)((tmpadress & 0xff00) >> 8);
            buffer[4] = Constants.TEMP_IDENT;
            char[] temp = textBox_TEMP_comm.Text.ToCharArray();
            buffer[5] = Convert.ToByte(temp[0]);
            buffer[6] = Convert.ToByte(temp[1]);
            if (textBox_TEMP_comm_CRC.Text != "")
            {
                buffer[7] = (byte)Convert.ToInt32(textBox_TEMP_comm_CRC.Text, 16);
                buffer[8] = 0x03;
                if (Command_Flag == 0)
                {
                    if (serialPort1.IsOpen)
                    {
                        serialPort1.Write(buffer, 0, buffer.Length);
                        Command_Flag = (1 << 3);
                        timer1.Start();
                    }
                }
                else
                {
                    MessageBox.Show("Ein Kommando noch nicht besttigt");
                }
            }
        }

        private void button_Multi_crc_Click(object sender, EventArgs e)
        {
            byte[] buffer = new byte[7];
            buffer[0] = 0x02;
            buffer[1] = 0xA3;
            int tmpadress = cmd_base_address + Constants.MULTI_CMD_ADDRESS;
            buffer[2] = (byte)(tmpadress & 0x00ff);
            buffer[3] = (byte)((tmpadress & 0xff00) >> 8);
            buffer[4] = Constants.MULTI_IDENT;
            char[] temp = textBox_MULTI_comm.Text.ToCharArray();
            buffer[5] = Convert.ToByte(temp[0]);
            buffer[6] = Convert.ToByte(temp[1]);
            int temp1 = Checksum(buffer);
            textBox_MULTI_comm_CRC.Text = temp1.ToString("X2");
            checkBox_MULTI_ack.Checked = false;
        }

        private void button_MULTI_send_Click(object sender, EventArgs e)
        {
            byte[] buffer = new byte[9];
            buffer[0] = 0x02;
            buffer[1] = 0xA3;
            int tmpadress = cmd_base_address + Constants.MULTI_CMD_ADDRESS;
            buffer[2] = (byte)(tmpadress & 0x00ff);
            buffer[3] = (byte)((tmpadress & 0xff00) >> 8);
            buffer[4] = Constants.MULTI_IDENT;
            char[] temp = textBox_MULTI_comm.Text.ToCharArray();
            buffer[5] = Convert.ToByte(temp[0]);
            buffer[6] = Convert.ToByte(temp[1]);
            if (textBox_MULTI_comm_CRC.Text != "")
            {

                buffer[7] = (byte)Convert.ToInt32(textBox_MULTI_comm_CRC.Text, 16);
                buffer[8] = 0x03;
                if (Command_Flag == 0)
                {
                    if (serialPort1.IsOpen)
                    {
                        serialPort1.Write(buffer, 0, buffer.Length);
                        Command_Flag = (1 << 4);
                        timer1.Start();
                    }
                }
                else
                {
                    MessageBox.Show("Ein Kommando noch nicht besttigt");
                }
            }
        }

        private void button_send_electrodes_Click(object sender, EventArgs e)
        {
            byte[] buffer = new byte[9];
            byte temp;

            buffer[0] = 0x02;
            buffer[1] = 0xA3;
            int tmpadress = cmd_base_address + Constants.ECG_CMD_ADDRESS;
            buffer[2] = (byte)(tmpadress & 0x00ff);
            buffer[3] = (byte)((tmpadress & 0xff00) >> 8);
            buffer[4] = Constants.ECG_IDENT;
            buffer[5] = Convert.ToByte('C');
            temp = 0;
            if (checkR.Checked)
                temp += 0x80;
            if (checkC.Checked)
                temp += 0x40;
            if (checkaVF.Checked)
                temp += 0x20;
            if (checkaVL.Checked)
                temp += 0x10;
            if (checkaVR.Checked)
                temp += 0x08;
            if (checkIII.Checked)
                temp += 0x04;
            if (checkII.Checked)
                temp += 0x02;
            if (checkI.Checked)
                temp += 0x01;
            buffer[6] = Convert.ToByte(temp);
            if (textBox_ECG_comm_CRC.Text != "")
            {

                buffer[7] = (byte)Convert.ToInt32(textBox_ECG_comm_CRC.Text, 16);
                buffer[8] = 0x03;
                if (Command_Flag == 0)
                {
                    if (serialPort1.IsOpen)
                    {
                        serialPort1.Write(buffer, 0, buffer.Length);
                        Command_Flag = (1 << 0);
                        timer1.Start();
                        ECG_ch_status_receive = false;

                    }
                }
                else
                {
                    MessageBox.Show("Ein Kommando noch nicht besttigt");
                }
                
            }

        }

        private void button_CRC_electrodes_Click(object sender, EventArgs e)
        {
            byte[] buffer = new byte[7];
            byte temp;
            int tmp_cnt = 0;

            buffer[0] = 0x02;
            buffer[1] = 0xA3;
            int tmpadress = cmd_base_address + Constants.ECG_CMD_ADDRESS;
            buffer[2] = (byte)(tmpadress & 0x00ff);
            buffer[3] = (byte)((tmpadress & 0xff00) >> 8);
            buffer[4] = Constants.ECG_IDENT;
            buffer[5] = Convert.ToByte('C');
            temp = 0;
            if (checkR.Checked)
            {
                temp += 0x80;
                tmp_cnt++;
            }
            if (checkC.Checked)
            {
                temp += 0x40;
                tmp_cnt++;
            }
            if (checkaVF.Checked)
            {
                temp += 0x20;
                tmp_cnt++;
            }
            if (checkaVL.Checked)
            {
                temp += 0x10;
                tmp_cnt++;
            }
            if (checkaVR.Checked)
            {
                temp += 0x08;
                tmp_cnt++;
            }
            if (checkIII.Checked)
            {
                temp += 0x04;
                tmp_cnt++;
            }
            if (checkII.Checked)
            {
                temp += 0x02;
                tmp_cnt++;
            }
            if (checkI.Checked)
            {
                temp += 0x01;
                tmp_cnt++;
            }
            buffer[6] = temp;
            int temp1 = Checksum(buffer);
            textBox_ECG_comm_CRC.Text = temp1.ToString("X2");
            checkBox_ECG_ack.Checked = false;

            //cnt_Channel = tmp_cnt;

        }

        private void TXD_CheckedChanged(object sender, EventArgs e)
        {
            checkBox_MULTI_ack.Checked = false;
            byte[] buffer = new byte[7];
            buffer[0] = 0x02;
            buffer[1] = 0xA3;
            int tmpadress = cmd_base_address + Constants.MULTI_TXD_ADDRESS;
            buffer[2] = (byte)(tmpadress & 0x00ff);
            buffer[3] = (byte)((tmpadress & 0xff00) >> 8);
            buffer[4] = Constants.MULTI_IDENT;
            //char[] temp = textBox_MULTI_comm.Text.ToCharArray();
            buffer[5] = Convert.ToByte('T');
            if (((Button)(sender)).Text == "TXDON")
            {


                buffer[6] = Convert.ToByte('1');
                //if (textBox_MULTI_comm_CRC.Text != "")
            }
            else
            {
                buffer[6] = Convert.ToByte('0');
            }

            byte temp1 = Checksum(buffer);
            byte[] tmpbuffer = new byte[9];
            Buffer.BlockCopy(buffer, 0, tmpbuffer, 0, buffer.Length);

            tmpbuffer[7] = (temp1);
            tmpbuffer[8] = 0x03;
           
            if (Command_Flag == 0)
            {
                if ((TX_status == true && buffer[6] == 0x30) || (TX_status == false && buffer[6] == 0x31))
                {
                    TX_status ^= true;
                    if (serialPort1.IsOpen)
                    {
                        serialPort1.Write(tmpbuffer, 0, tmpbuffer.Length);
                        Command_Flag = (1 << 4);
                        timer1.Start();
                    }
                }
            }
            else
            {
                MessageBox.Show("Ein Kommando noch nicht besttigt");
            }
                
            
        }

        private void textBox_ECG_base_address_TextChanged(object sender, EventArgs e)
        {
            
            ecg_base_address = Convert.ToInt16(textBox_ECG_base_address.Text,16);
        }

        private void textBox_base_address_TextChanged(object sender, EventArgs e)
        {
            base_address = Convert.ToInt16(textBox_base_address.Text,16);
        }

        private void textBox_CMD_base_adress_TextChanged(object sender, EventArgs e)
        {
            cmd_base_address = Convert.ToInt16(textBox_CMD_base_address.Text,16);
        }





        public void ECG_nextLimb(byte[] buffer)
        {

            int lng = (buffer[1] & 0x0F)+ 4;
            
            {
                for (int i = 4; i < lng; i++)
                {
                    Datensatz[i - 4] = buffer[i];
                }

                ECG_Display();
                if (index % ((this.pictureBox_Graph.Image.Width / column - 35) / 10) == 0)
                {
                    Graphics g = Graphics.FromImage(this.pictureBox_Graph.Image);
                    //dumint = rand + ((int)(Math.Round((double)((tempxrand - rand) * (j / 10.0)))));
                    g.DrawString(DateTime.Now.ToLongTimeString(), new Font(FontFamily.GenericSerif, 8, FontStyle.Bold), new SolidBrush(Color.Black), new Point((index), this.pictureBox_Graph.Height - 20));
                    //form1.pictureBox_Graph.Refresh();
                    //g.DrawLine(blackPen, new Point(dumint, ch._base - ch._hoehe + gr_rand), new Point(dumint, ch._base));
                }
                if (this.pictureBox_Graph.Image != null)
                {
                    if (index < (this.pictureBox_Graph.Image.Width / column - 10))
                    {
                        index = index + 1;
                    }
                    else
                    {
                        index = rand + 2;
                        display_diag_exit = false;
                        ECG_display_diag();
                    }
                }
                 if (save == true)
                    {
                        if (outfileCtr > 0)
                        {
                            int j = 0;
                            
                            outfile.Write(outfileCtr);
                            outfile.Write("\t");                        
                            foreach (channel Member in Channel)
                                {
                                    Control[] temp = panel6.Controls.Find(("check"+Member.CH_name), true);
                                    if (temp.Length>0)
                                    {
                                        if (Member._hoehe > 0)
                                        {
                                            outfile.Write(Datensatz[j++]);
                                            outfile.Write("\t");
                                        }
                                        else
                                        {
                                            outfile.Write(0);
                                            outfile.Write("\t");
                                        }
                                    }
                                }
                            }
                            outfile.Write(Convert.ToString(ecg_pulse));
                            outfile.Write("\t");
                            outfile.Write(Convert.ToString( ecg_resp));
                            outfile.Write("\t");
                            outfile.WriteLine(ecg_Status);
                            
                            outfileCtr++;
                        }

            }
        }

        private int ECG_display_wave(channel ch, int i)
        {

            int dumint;
            double factor = (ch._hoehe - gr_rand) / 256.0;

            Graphics g = Graphics.FromImage(this.pictureBox_Graph.Image);
            Pen drawPen = new Pen(Color.Black, 1);
            drawPen.DashStyle = DashStyle.Solid;

            drawPen.Color = ch.Color;
            dumint = ch._base - (int)Math.Round((double)(Datensatz[i++] * factor));
            if ((packetsread > 3) && (display_diag_exit == true)) //(* 1. Datensatz nicht anzeigen	*)
            {
                if (ch.xoffset == true)
                {
                    g.DrawLine(drawPen, new Point(index - 1 + this.pictureBox_Graph.Width / 2, ch.old_wave), new Point(index + this.pictureBox_Graph.Width / 2, dumint));
                }
                else
                {
                    g.DrawLine(drawPen, new Point(index - 1, ch.old_wave), new Point(index, dumint));
                }
            }
            ch.old_wave = dumint;

            g.Dispose();
            drawPen.Dispose();

            return i;
        }

        // Draw Graphs and save Value
        private void ECG_Display()
        {

            //bmp = (Bitmap)form1.pictureBox_Graph.Image;

            if ((this.pictureBox_Graph.Image != null))
            {
                int i = 0;


                packetsread += 1;
                if (packetsread > 5)
                {
                    packetsread = 5;
                }
                foreach (channel Member in Channel)
                {
                    if (Member._hoehe > 0)
                    {
                        i = ECG_display_wave(Member, i);
                    }

                }
                

            }
        }

        // Refresh Picturebox of graph
        public void ECG_update()
        {
            if ((display_diag_exit == true) && (Status_ok == true))
            {

                this.pictureBox_Graph.Invalidate();


            }
        }
        
        bool display_diag_exit = false;

        Bitmap bmp1;

        // Funktion to draw diagrammlegend
        private void ECG_draw_Diagramm(channel ch)
        {
            PictureBox PicB = this.pictureBox_Graph;

            Graphics g = Graphics.FromImage(bmp1);

            Pen blackPen = new Pen(Color.Black, 1);
            int dumint;
            int tempxrand = ((PicB.Width / column) - 10);

            if (ch._hoehe > 0) 		//				(* Achsen zeichnen	*)		
            {
                blackPen.DashStyle = DashStyle.Solid;
                blackPen.Color = Color.Black;

                if (ch.xoffset == true)
                {

                    g.DrawLine(blackPen, new Point(rand + (PicB.Width / column), ch._base), new Point(((PicB.Width) - 10), ch._base));
                    g.DrawLine(blackPen, new Point(rand + (PicB.Width / column), ch._base - ch._hoehe + gr_rand), new Point(rand + (PicB.Width / column), ch._base));

                    g.DrawString(ch.CH_name, new Font(FontFamily.GenericSerif, 10, FontStyle.Bold), new SolidBrush(Color.Black), new Point(rand + (PicB.Width / column), ch._base - ch._hoehe + 4));


                    blackPen.DashStyle = DashStyle.Dot;
                    blackPen.Color = Color.Gray;
                    for (int j = 1; j < 4; j++)
                    {
                        dumint = (int)(Math.Round((ch._base - (ch._hoehe - gr_rand) * (0.25 * j)), 0));
                        g.DrawLine(blackPen, new Point(rand + (PicB.Width / column), dumint), new Point(((PicB.Width) - 10), dumint));

                    }
                    for (int j = 1; j < 10; j++)
                    {
                        dumint = rand + (PicB.Width / column) + ((int)(Math.Round((double)((tempxrand - rand) * (j / 10.0)))));
                        g.DrawLine(blackPen, new Point(dumint, ch._base - ch._hoehe + gr_rand), new Point(dumint, ch._base));
                    }
                }
                else
                {

                    g.DrawLine(blackPen, new Point(rand, ch._base), new Point(tempxrand, ch._base));
                    g.DrawLine(blackPen, new Point(rand, ch._base - ch._hoehe + gr_rand), new Point(rand, ch._base));

                    g.DrawString(ch.CH_name, new Font(FontFamily.GenericSerif, 10, FontStyle.Bold), new SolidBrush(Color.Black), new Point(rand, ch._base - ch._hoehe + 4));


                    blackPen.DashStyle = DashStyle.Dot;
                    blackPen.Color = Color.Gray;
                    for (int j = 1; j < 4; j++)
                    {
                        dumint = (int)(Math.Round((ch._base - (ch._hoehe - gr_rand) * (0.25 * j)), 0));
                        g.DrawLine(blackPen, new Point(rand, dumint), new Point(tempxrand, dumint));

                    }
                    for (int j = 1; j < 10; j++)
                    {
                        dumint = rand + ((int)(Math.Round((double)((tempxrand - rand) * (j / 10.0)))));
                        g.DrawLine(blackPen, new Point(dumint, ch._base - ch._hoehe + gr_rand), new Point(dumint, ch._base));
                    }
                }

            }
            g.Dispose();
            //PicB.Dispose();
            blackPen.Dispose();

        }

        // Draw all Diagramms
        public void ECG_display_diag()
        { 
            PictureBox PicB = this.pictureBox_Graph;

             

            int temp_Ch = cnt_Channel;

            display_diag_exit = false;

            if (temp_Ch > 8)
            {
                temp_Ch = 8;
            }
            int Diagram_height = 0;
            if (temp_Ch > 0)
            {
                Diagram_height = ((PicB.Height - rand) / temp_Ch);
            }


            if (temp_Ch > 0)
            {
                if (this.checkI.Checked)
                {
                    Channel[(int)cha.chi]._base = Diagram_height;
                    Channel[(int)cha.chi]._hoehe = Channel[(int)cha.chi]._base;
                }
                else
                {
                    Channel[(int)cha.chi]._base = 0;
                    Channel[(int)cha.chi]._hoehe = 0;
                }

                if (this.checkII.Checked)
                {
                    Channel[(int)cha.chii]._base = Channel[(int)cha.chi]._base + Diagram_height;
                    Channel[(int)cha.chii]._hoehe = Diagram_height;
                }
                else
                {
                    Channel[(int)cha.chii]._hoehe = 0;
                    Channel[(int)cha.chii]._base = Channel[(int)cha.chi]._base;
                }

                if (this.checkIII.Checked)
                {
                    Channel[(int)cha.chiii]._base = Channel[(int)cha.chii]._base + Diagram_height;
                    Channel[(int)cha.chiii]._hoehe = Diagram_height;
                }
                else
                {
                    Channel[(int)cha.chiii]._hoehe = 0;			//(* nicht darstellen	*)
                    Channel[(int)cha.chiii]._base = Channel[(int)cha.chii]._base;
                }

                if (this.checkaVR.Checked)
                {
                    Channel[(int)cha.chvr]._base = Channel[(int)cha.chiii]._base + Diagram_height;
                    Channel[(int)cha.chvr]._hoehe = Diagram_height;
                }
                else
                {
                    Channel[(int)cha.chvr]._hoehe = 0;			//(* nicht darstellen	*)
                    Channel[(int)cha.chvr]._base = Channel[(int)cha.chiii]._base;
                }
                if (this.checkaVL.Checked)
                {
                    Channel[(int)cha.chvl]._base = Channel[(int)cha.chvr]._base + Diagram_height;
                    Channel[(int)cha.chvl]._hoehe = Diagram_height;
                }
                else
                {
                    Channel[(int)cha.chvl]._hoehe = 0;			//(* nicht darstellen	*)
                    Channel[(int)cha.chvl]._base = Channel[(int)cha.chvr]._base;
                }

                if (this.checkaVF.Checked)
                {
                    Channel[(int)cha.chvf]._base = Channel[(int)cha.chvl]._base + Diagram_height;
                    Channel[(int)cha.chvf]._hoehe = Diagram_height;
                }
                else
                {
                    Channel[(int)cha.chvf]._hoehe = 0;			//(* nicht darstellen	*)
                    Channel[(int)cha.chvf]._base = Channel[(int)cha.chvl]._base;
                }
                if (this.checkC.Checked)
                {
                    Channel[(int)cha.chc1]._base = Channel[(int)cha.chvf]._base + Diagram_height;
                    Channel[(int)cha.chc1]._hoehe = Diagram_height;
                }
                else
                {
                    Channel[(int)cha.chc1]._hoehe = 0;			//(* nicht darstellen	*)
                    Channel[(int)cha.chc1]._base = Channel[(int)cha.chvf]._base;
                }

                
                if (this.checkR.Checked)
                {
                    if ((Channel[(int)cha.chc1]._base + Diagram_height) > PicB.Height)
                    {
                        Channel[(int)cha.chresp]._base = 0 + Diagram_height;
                        Channel[(int)cha.chresp].xoffset = true;


                    }
                    else
                    {
                        Channel[(int)cha.chresp]._base = Channel[(int)cha.chc1]._base + Diagram_height;
                        if (Channel[(int)cha.chc1].xoffset == true)
                        {
                            Channel[(int)cha.chresp].xoffset = true;
                        }
                        else
                        {
                            Channel[(int)cha.chresp].xoffset = false;
                        }
                    }
                    Channel[(int)cha.chresp]._hoehe = Diagram_height;

                }
                else
                {
                    Channel[(int)cha.chresp]._hoehe = 0;			//(* nicht darstellen	*)
                    Channel[(int)cha.chresp]._base = Channel[(int)cha.chc1]._base;
                    if (Channel[(int)cha.chc1].xoffset == true)
                    {
                        Channel[(int)cha.chresp].xoffset = true;
                    }
                    else
                    {
                        Channel[(int)cha.chresp].xoffset = false;
                    }

                }



                if (PicB.Height > 0)
                {
                    bmp1 = new Bitmap(PicB.Width, PicB.Height);

                    Graphics g = Graphics.FromImage(bmp1);

                    Pen blackPen = new Pen(Color.Black, 1);
                    g.Clear(Color.White);
                    //string tempstr = max_Wert.ToString() + " mv";
                    //g.DrawLine(blackPen, new Point(rand-10,  gr_rand), new Point(rand-10,Diagram_height));
                    //g.DrawString("0 mv", new Font(FontFamily.GenericSerif, 8, FontStyle.Regular), new SolidBrush(Color.Black), new Point(3, Diagram_height - 5));
                    //g.DrawString(tempstr, new Font(FontFamily.GenericSerif, 8, FontStyle.Regular), new SolidBrush(Color.Black), new Point(3, gr_rand - 5));

                    foreach (channel Member in Channel)
                    {
                        ECG_draw_Diagramm(Member);
                    }


                    this.pictureBox_Graph.Image = bmp1;
                    this.pictureBox_Graph.Refresh();
                    display_diag_exit = true;
                    g.Dispose();
                    //PicB.Dispose();
                    blackPen.Dispose();
                }
            }
        }

        int[] Datensatz= new int [13];        // Array of last Wavevalues
        int packetsread = 0;
        int column = 1;         // Column of the graph
        byte rand = 30;
        int index;
        byte gr_rand = 20;
        int max_Wert=8;

        bool Status_ok = true;

        public class channel
        {
            public string CH_name;
            public int _hoehe;
            public int _base;
            public Color Color;
            public bool xoffset;
            public int old_wave;

            public channel(string n, int h, int b, Color col, bool off)
            {
                CH_name = n;
                _hoehe = h;
                _base = b;
                Color = col;
                xoffset = off;
                old_wave = 0;
            }
        }
        int cnt_Channel = 0;
        string[] Signal = { "I", "II", "III", "aVR", "aVL", "aVF", "C", 
                                "R" };

        enum cha { chi, chii, chiii, chvr, chvl, chvf, chc1, chresp };
        channel[] Channel = new channel[]
            {
                new channel("I",0, 0, Color.Blue, false),
                new channel("II",0, 0, Color.Brown, false),
                new channel("III",0, 0, Color.Green, false),
                new channel("aVR",0, 0, Color.Black , false),
                new channel("aVL",0, 0, Color.Blue, false),
                new channel("aVF",0, 0, Color.Indigo, false),
                new channel("C",0, 0, Color.Green, false),
                new channel("R",0, 0, Color.BlueViolet, false),
                
            };

        private void timer2_Tick(object sender, EventArgs e)
        {
            ECG_update();
            pictureBox1.Invalidate();
        }

        

        private void tabControl1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (tabControl1.SelectedIndex == 1)
            {
                ECG_display_diag();
                packetsread = 0;
                index = 30;
            }
        }

        private void SPO_Display(int wave)
        {

            if (SPO_Status != 0)
            {
                wave = 128;
            }

            //bar1.Value = wave;

            if (index2 < pictureBox1.Width)
            {
                index2 = index2 + 2;
            }
            else
            {
                index2 = 0;
            }
            double fact = (double)pictureBox1.Height / 256.0;
            wave = (int)(pictureBox1.Height - (wave*fact));

            g.DrawLine(new Pen(Color.White, 2), index2, 1, index2, pictureBox1.Height);
            g.DrawLine(new Pen(Color.White, 2), index2 + 1, 1, index2 + 1, pictureBox1.Height);

            if (index >= 1)
            {
                g.DrawLine(new Pen(Color.Blue, 1), index2 - 2, old_wave, index2, wave);

            }

            old_wave = wave;

            //pictureBox1.Image = bmp;
            //pictureBox1.Invalidate();



        }
        int SPO_Status = 0;
        int index2 = 0;
        int old_wave = 0;
        Graphics g;
        Bitmap bmp;

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AboutBox frm = new AboutBox();
            frm.Show();
        }

        private void button_CO2_CRC_Click(object sender, EventArgs e)
        {
            byte[] buffer = new byte[7];
            buffer[0] = 0x02;
            buffer[1] = 0xA3;
            int tmpadress = cmd_base_address + Constants.CO2_CMD_ADDRESS;
            buffer[2] = (byte)(tmpadress & 0x00ff);
            buffer[3] = (byte)((tmpadress & 0xff00) >> 8);
            buffer[4] = Constants.CO2_IDENT;
            char[] temp = textBox_Co2_comm.Text.ToCharArray();
            buffer[5] = Convert.ToByte(temp[0]);
            buffer[6] = Convert.ToByte(temp[1]);
            int temp1 = Checksum(buffer);
            textBox_Co2_comm_CRC.Text = temp1.ToString("X2");
            checkBox_CO2_Ack.Checked = false;
        }

        private void button_CO2_send_Click(object sender, EventArgs e)
        {
            byte[] buffer = new byte[9];
            buffer[0] = 0x02;
            buffer[1] = 0xA3;
            int tmpadress = cmd_base_address + Constants.CO2_CMD_ADDRESS;
            buffer[2] = (byte)(tmpadress & 0x00ff);
            buffer[3] = (byte)((tmpadress & 0xff00) >> 8);
            buffer[4] = Constants.CO2_IDENT;
            char[] temp = textBox_Co2_comm.Text.ToCharArray();
            buffer[5] = Convert.ToByte(temp[0]);
            buffer[6] = Convert.ToByte(temp[1]);
            if (textBox_Co2_comm_CRC.Text != "")
            {
                buffer[7] = (byte)Convert.ToInt32(textBox_Co2_comm_CRC.Text, 16);
                //buffer[7] = Checksum(buffer);
                buffer[8] = 0x03;
                if (Command_Flag == 0)
                {
                    if (serialPort1.IsOpen)
                    {
                        serialPort1.Write(buffer, 0, buffer.Length);
                        Command_Flag = (1 << 2);
                        timer1.Start();
                    }
                }
                else
                {
                    MessageBox.Show("Ein Kommando noch nicht besttigt");
                }
            }
        }

        private void VersionToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (((ToolStripMenuItem)sender).Text == "NIBP")
            {
                nIBPToolStripMenuItem.Checked = true;
                cO2ToolStripMenuItem.Checked = false;
                iBPToolStripMenuItem.Checked = false;
                panel_CO2.Visible = false;
                panel_NIBP.Visible = true;
                panel_IBP.Visible = false;
                panel_IPB2.Visible = false;
            }
            else if (((ToolStripMenuItem)sender).Text == "CO2")
            {
                nIBPToolStripMenuItem.Checked = false;
                cO2ToolStripMenuItem.Checked = true;
                iBPToolStripMenuItem.Checked = false;
                panel_CO2.Visible = true;
                panel_NIBP.Visible = false;
                panel_IBP.Visible = false;
                panel_IPB2.Visible = false;
            }
            else if (((ToolStripMenuItem)sender).Text == "IBP")
            {
                nIBPToolStripMenuItem.Checked = false;
                cO2ToolStripMenuItem.Checked = false;
                iBPToolStripMenuItem.Checked = true;
                panel_CO2.Visible = false;
                panel_NIBP.Visible = false;
                panel_IBP.Visible = true;
                panel_IPB2.Visible = true;
            }
        }

        private void button_IBP1_CRC_Click(object sender, EventArgs e)
        {
            byte[] buffer = new byte[7];
            buffer[0] = 0x02;
            buffer[1] = 0xA3;
            int tmpadress = cmd_base_address + Constants.IBP1_CMD_ADDRESS;
            buffer[2] = (byte)(tmpadress & 0x00ff);
            buffer[3] = (byte)((tmpadress & 0xff00) >> 8);
            buffer[4] = Constants.IBP1_IDENT;
            char[] temp = textBox_IBP1_comm.Text.ToCharArray();
            buffer[5] = Convert.ToByte(temp[0]);
            buffer[6] = Convert.ToByte(temp[1]);
            int temp1 = Checksum(buffer);
            textBox_IBP1_comm_CRC.Text = temp1.ToString("X2");
            checkBox_IBP1_ack.Checked = false;
        }

        private void button_IBP1_send_Click(object sender, EventArgs e)
        {
            byte[] buffer = new byte[9];
            buffer[0] = 0x02;
            buffer[1] = 0xA3;
            int tmpadress = cmd_base_address + Constants.IBP1_CMD_ADDRESS;
            buffer[2] = (byte)(tmpadress & 0x00ff);
            buffer[3] = (byte)((tmpadress & 0xff00) >> 8);
            buffer[4] = Constants.IBP1_IDENT;
            char[] temp = textBox_IBP1_comm.Text.ToCharArray();
            buffer[5] = Convert.ToByte(temp[0]);
            buffer[6] = Convert.ToByte(temp[1]);
            if (textBox_IBP1_comm_CRC.Text != "")
            {
                buffer[7] = (byte)Convert.ToInt32(textBox_IBP1_comm_CRC.Text, 16);
                //buffer[7] = Checksum(buffer);
                buffer[8] = 0x03;
                if (Command_Flag == 0)
                {
                    if (serialPort1.IsOpen)
                    {
                        serialPort1.Write(buffer, 0, buffer.Length);
                        Command_Flag = (1 << 2);
                        timer1.Start();
                    }
                }
                else
                {
                    MessageBox.Show("Ein Kommando noch nicht besttigt");
                }
            }
        }

        private void button_IBP2_CRC_Click(object sender, EventArgs e)
        {
            byte[] buffer = new byte[7];
            buffer[0] = 0x02;
            buffer[1] = 0xA3;
            int tmpadress = cmd_base_address + Constants.IBP2_CMD_ADDRESS;
            buffer[2] = (byte)(tmpadress & 0x00ff);
            buffer[3] = (byte)((tmpadress & 0xff00) >> 8);
            buffer[4] = Constants.IBP2_IDENT;
            char[] temp = textBox_IBP2_comm.Text.ToCharArray();
            buffer[5] = Convert.ToByte(temp[0]);
            buffer[6] = Convert.ToByte(temp[1]);
            int temp1 = Checksum(buffer);
            textBox_IBP2_comm_CRC.Text = temp1.ToString("X2");
            checkBox_IBP2_ack.Checked = false;
        }

        private void button_IBP2_send_Click(object sender, EventArgs e)
        {
            byte[] buffer = new byte[9];
            buffer[0] = 0x02;
            buffer[1] = 0xA3;
            int tmpadress = cmd_base_address + Constants.IBP2_CMD_ADDRESS;
            buffer[2] = (byte)(tmpadress & 0x00ff);
            buffer[3] = (byte)((tmpadress & 0xff00) >> 8);
            buffer[4] = Constants.IBP2_IDENT;
            char[] temp = textBox_IBP2_comm.Text.ToCharArray();
            buffer[5] = Convert.ToByte(temp[0]);
            buffer[6] = Convert.ToByte(temp[1]);
            if (textBox_IBP2_comm_CRC.Text != "")
            {
                buffer[7] = (byte)Convert.ToInt32(textBox_IBP2_comm_CRC.Text, 16);
                //buffer[7] = Checksum(buffer);
                buffer[8] = 0x03;
                if (Command_Flag == 0)
                {
                    if (serialPort1.IsOpen)
                    {
                        serialPort1.Write(buffer, 0, buffer.Length);
                        Command_Flag = (1 << 2);
                        timer1.Start();
                    }
                }
                else
                {
                    MessageBox.Show("Ein Kommando noch nicht besttigt");
                }
            }
        }
        StreamWriter outfile;
        string File = null;
        int outfileCtr = -1;

        private void button_rec_Click(object sender, EventArgs e)
        {
            if (((Button)sender).Name == "button_rec")
            {
                if (serialPort1.IsOpen)
                {
                    SaveFileDialog dialog = new SaveFileDialog();
                    dialog.Filter = "Log files (*.log)|*.log|txt files (*.txt)|*.txt|All files (*.*)|*.*";
                    dialog.Title = "Select a file";
                    //dialog.ShowDialog();


                    if (dialog.ShowDialog() == DialogResult.OK)
                    {
                        File = dialog.FileName;
                        //ekg.outfile = new StreamWriter(dialog.FileName);
                        //ekg.outfileCtr = 1;

                        toolStripStatusLabel1.Text = dialog.FileName;
                    }
                    else
                    {

                    }




                    if (outfile == null)
                    {
                        if (File != null)
                        {

                            //ekg.outfileCtr = 1;
                            try
                            {
                                outfile = new StreamWriter(File);
                            }
                            catch
                            {
                                MessageBox.Show("Datei " + File +
                                        " cannot be opened!", "Datei fehler",
                                        MessageBoxButtons.OK, MessageBoxIcon.Warning);
                            }
                        }
                    }

                    if (outfile != null)
                    {
                        okayToolStripMenuItem.Checked = true;
                        outfileCtr = 1;

                        outfile.WriteLine("EG05000");
                        outfile.WriteLine( DateTime.Now.ToLongDateString() + " " + DateTime.Now.ToLongTimeString());
                        outfile.WriteLine("\"Count\"\t\"I\"\t\"II\"\t\"III\"\t\"aVR\"\t\"aVL\"\t\"aVF\"\t\"C1\"\t\"Resp\"\t\"Puls\"\t\"Rate\"\t\"Status1\"\t\"Status2\"\t\"Status3\"\t\"Status4\"");

                        save = true;
                    }
                }
            }
            else if (((Button)sender).Name == "button_stop")
            {
                //okayToolStripMenuItem.Checked = false;
                save = false;
                outfileCtr = 0;
                if (outfile != null)
                {
                    outfile.Close();
                    outfile = null;
                }
            }
            
        }
    }
}